﻿package {
	import flash.events.*;
	public class Reversi extends EventDispatcher {
		static public const EMPTY:uint  = 0;
		static public const COLOR1:uint = 1;
		static public const COLOR2:uint = 2;

		static public const BOARD_WIDTH:uint = 8;
		static public const BOARD_HEIGHT:uint = 8;

		public var fieldStatus:Array;
		public var nowPlayer:uint = 0;
		
		function Reversi(){}

		// 初期化
		public function initialize():void {
			this.fieldStatus = new Array();
			for (var x:int = 0; x < BOARD_WIDTH; x++) {
				this.fieldStatus[x] = new Array();
				for (var y:int = 0; y < BOARD_HEIGHT; y++) {
					this.fieldStatus[x][y] = EMPTY;
				}
			}
			setStone(3, 3, COLOR1, false);
			setStone(4, 4, COLOR1, false);
			setStone(4, 3, COLOR2, false);
			setStone(3, 4, COLOR2, false);
		}
		
		// プレイヤー交代
		public function playerChange(color:uint = 0):void {
			this.nowPlayer = color;
			var eve:DynamicEvent = new DynamicEvent("playerChange");
			eve.color = color;
			this.dispatchEvent(eve);
		}
		
		
		// 石をセット
		public function setStone(x:int, y:int, color:uint, bSpin:Boolean = true):void {
			this.fieldStatus[x][y] = color;
			var eve:DynamicEvent = new DynamicEvent("stoneSet");
			eve.x = x;
			eve.y = y;
			eve.color = color;
			this.dispatchEvent(eve);

			// 石を回転する
			if (bSpin) {
				for (var sx:int = -1; sx <= 1; sx++) {
					for (var sy:int = -1; sy <= 1; sy++) {
						if (sx == 0 && sy == 0) continue;
						// 取れる数をチェック
						var getStones:uint = this.getStoneCount(x, y, sx, sy, color);
						if (getStones > 0) {
							for(var i:uint = 1; i <= getStones; i++) {
								spinStone(x + sx * i, y + sy * i, color);
							}
						}
					}
				}
			}
		}
		
		// 石を回転
		public function spinStone(x:int, y:int, color:uint):void {
			this.fieldStatus[x][y] = color;
			var eve:DynamicEvent = new DynamicEvent("stoneSpin");
			eve.x = x;
			eve.y = y;
			eve.color = color;
			this.dispatchEvent(eve);
		}
		
		// 石を取得
		public function getStone(x:int, y:int):uint {
			return this.fieldStatus[x][y] || EMPTY;
		}
		
		// 空のフィールド？
		public function isEmptyField(x:int, y:int):Boolean {
			return this.getStone(x, y) == EMPTY;
		}
		
		// フィールドエリア内？
		public function inField(x:int, y:int):Boolean {
			return (x >= 0 && y >= 0 && x < BOARD_WIDTH && y < BOARD_HEIGHT);
		}		
		
		// 設置可能な場所を取得
		public function getAvailableFields(color:uint):Array {
			var available:Array = new Array();
			for (var x:uint = 0; x < BOARD_WIDTH; x++) {
				for (var y:uint = 0; y < BOARD_HEIGHT; y++) {
					var getStones:uint = this.isAvailableField(x, y, color);
					if (getStones > 0) {
						available.push({x:x, y:y, count:getStones});
					}
				}
			}
			return available;
		}
		
		// 設置可能な場所かチェック
		public function isAvailableField(x:int, y:int, myColor:uint):uint {
			var getStones:uint = 0;
			if (!this.isEmptyField(x, y)) return 0;
			
			for (var sx:int = -1; sx <= 1; sx++) {
				for (var sy:int = -1; sy <= 1; sy++) {
					if (sx == 0 && sy == 0) continue;
					getStones += this.getStoneCount(x, y, sx, sy, myColor);
				}
			}
			return getStones;
		}
		
		public function getStoneCount(x:int, y:int, sx:int, sy:int, myColor:uint):uint {
			var getStones:uint = 0;
			while(1) {
				x += sx;
				y += sy;
				if (!this.inField(x, y)) return 0;
				var color:uint = this.getStone(x, y);
				if (color == EMPTY)   return 0;
				if (color == myColor) break;
				getStones++;
			}
			return getStones;
		}
		
		
	}	
}